@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2010-2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2010 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2011 Hans de Goede <hdegoede@redhat.com>
  *
  * SPDX-License-Identifier: LGPL-2.1+
@@ -12,7 +12,7 @@
 
 G_BEGIN_DECLS
 
-#define G_USB_SOURCE_ERROR			(g_usb_source_error_quark ())
+#define G_USB_SOURCE_ERROR (g_usb_source_error_quark())
 
 typedef struct _GUsbSource GUsbSource;
 
@@ -21,17 +21,14 @@ typedef struct _GUsbSource GUsbSource;
  *
  * The error code.
  **/
-typedef enum {
-	G_USB_SOURCE_ERROR_INTERNAL
-} GUsbSourceError;
+typedef enum { G_USB_SOURCE_ERROR_INTERNAL } GUsbSourceError;
 
 G_DEPRECATED_FOR(g_usb_context_error_quark)
-GQuark		 g_usb_source_error_quark	(void);
+GQuark
+g_usb_source_error_quark(void);
 
 G_DEPRECATED
-void		 g_usb_source_set_callback	(GUsbSource	*source,
-						 GSourceFunc	 func,
-						 gpointer	 data,
-						 GDestroyNotify	 notify);
+void
+g_usb_source_set_callback(GUsbSource *self, GSourceFunc func, gpointer data, GDestroyNotify notify);
 
 G_END_DECLS
